// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_text_format_argument.h
 *  \brief      MGL テキストフォーマットの引数
 *  \date       Since: May 28, 2021. 22:25:20 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_MGL_TEXT_FORMAT_ARGUMENT_H_1622208320
#define INCGUARD_MGL_TEXT_FORMAT_ARGUMENT_H_1622208320

#include <utility>

#include <mgl/file/mgl_file_path_view.h>
#include <mgl/stl/mgl_stl_containers.h>
#include <mgl/stl/mgl_stl_string.h>
#include <mgl/text/mgl_text_defs.h>
#include <mgl/text/mgl_text_format_options.h>

namespace MGL::Text
{
//! テキストフォーマットの引数
class FormatArgument
{
public:
    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument() noexcept
        : _type(Type::Void)
        , _i(0)
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(unsigned long long)
     *  \param[in]  value   符号なしの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(unsigned long long value) noexcept
        : _type(Type::UnsignedInt)
        , _ui(value)
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(unsigned long)
     *  \param[in]  value   符号なしの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(unsigned long value) noexcept
        : FormatArgument(static_cast<uint64_t>(value))
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(unsigned int)
     *  \param[in]  value   符号なしの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(unsigned int value) noexcept
        : FormatArgument(static_cast<uint64_t>(value))
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(unsigned short)
     *  \param[in]  value   符号なしの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(unsigned short value) noexcept
        : FormatArgument(static_cast<uint64_t>(value))
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(unsigned char)
     *  \param[in]  value   符号なしの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(unsigned char value) noexcept
        : FormatArgument(static_cast<uint64_t>(value))
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(long long)
     *  \param[in]  value   符号付きの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(long long value) noexcept
        : _type(Type::Int)
        , _i(value)
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(long)
     *  \param[in]  value   符号付きの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(long value) noexcept
        : FormatArgument(static_cast<int64_t>(value))
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(int)
     *  \param[in]  value   符号付きの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(int value) noexcept
        : FormatArgument(static_cast<int64_t>(value))
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(short)
     *  \param[in]  value   符号付きの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(short value) noexcept
        : FormatArgument(static_cast<int64_t>(value))
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(char)
     *  \param[in]  value   符号付きの値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(char value) noexcept
        : FormatArgument(static_cast<int64_t>(value))
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(float)
     *  \param[in]  value   浮動小数点数
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(float value) noexcept
        : _type(Type::Float)
        , _ui(0)
    {
        _f = value;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(double)
     *  \param[in]  value   浮動小数点数
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(double value) noexcept
        : _type(Type::Float)
        , _ui(0)
    {
        _f = static_cast<float>(value);
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(bool)
     *  \param[in]  value   論理値
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(bool value) noexcept
        : _type(Type::Bool)
        , _ui(0)
    {
        _b = value;
    }

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(STL::string)
     *  \param[in]  value   文字列
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(STL::string value) noexcept
        : _type(Type::String)
        , _i(0)
        , _string(std::move(value))
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(const char *)
     *  \param[in]  value   文字列
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(const char *value) noexcept
        : _type(Type::String)
        , _i(0)
        , _string(value)
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(File::PathView)
     *  \param[in]  value   文字列
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(const File::PathView &value) noexcept
        : _type(Type::String)
        , _i(0)
        , _string(value)
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(File::Path)
     *  \param[in]  value   文字列
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(const File::Path &value) noexcept
        : _type(Type::String)
        , _i(0)
        , _string(value)
    {}

    /* ------------------------------------------------------------------------- */
    /*!
     *  \brief      コンストラクタ(const void *)
     *  \param[in]  value   ポインタ変数
     *  \note
     *      アドレスを出力する．
     *      const char *型は文字列として認識されるため，void *型にキャストする必要がある．
     */
    /* ------------------------------------------------------------------------- */
    FormatArgument(const void *value) noexcept
        : _type(Type::Pointer)
        , _ui(reinterpret_cast<uint64_t>(value))
    {}

    [[nodiscard]] STL::string ToString(const FormatOptions &options = FormatOptions()) const noexcept;

    static const char *Parse(FormatOptions &options, const char *parseText) noexcept;

    static STL::vector<IndexedCharacter> ToIndexedCharacters(const FormatOptions &options) noexcept;

private:
    //! 値の型
    enum class Type : uint8_t
    {
        Void,           //!< 型なし
        Int,            //!< 符号付き整数
        UnsignedInt,    //!< 符号なし整数
        Float,          //!< 浮動小数点数
        Bool,           //!< 論理値
        String,         //!< 文字列
        Pointer,        //!< ポインタ
    };

    [[nodiscard]] bool IsZero() const noexcept;
    [[nodiscard]] bool IsPositiveValue() const noexcept;
    [[nodiscard]] bool IsNegativeValue() const noexcept;
    [[nodiscard]] STL::string ToStringFromNumber(const FormatOptions &options) const noexcept;
    [[nodiscard]] STL::string ToStringFromDecimal(const FormatOptions &options) const noexcept;
    [[nodiscard]] STL::string ToHexStringFromValue(const FormatOptions &options) const noexcept;

    static void AlignmentString(STL::string &result, const FormatOptions &options, const STL::string &string) noexcept;

    Type _type;
    union
    {
        int64_t _i;
        uint64_t _ui;
        float _f;
        bool _b;
    };

    STL::string _string;
};

//! テキストフォーマットの引数の配列
using FormatArgs = STL::vector<FormatArgument>;

}    // namespace MGL::Text

#endif    // INCGUARD_MGL_TEXT_FORMAT_ARGUMENT_H_1622208320

// vim: et ts=4 sw=4 sts=4
