// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       mgl_file_throwing_utility.cc
 *  \brief      MGL 例外を発生させるファイルユーティリティ
 *  \date       Since: September 29, 2022. 15:47:34 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include <mgl/file/mgl_file_throwing_utility.h>

#include <mgl/file/mgl_file_accessor.h>

namespace MGL::File
{
/* ------------------------------------------------------------------------- */
/*!
 *  \brief      ファイルのサイズを取得
 *  \param[in]  path                サイズを取得するファイルのパス
 *  \return     ファイルサイズ
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
size_t ThrowingUtility::GetSize(const PathView &path)
{
    auto size = Accessor::GetSize(_result, path);

    if (_result.HasError())
    {
        throw _result;
    }

    return size;
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      ディレクトリを作成する
 *  \param[in]  path                作成するディレクトリのパス
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
void ThrowingUtility::MakeDirectory(const PathView &path)
{
    _result = Accessor::MakeDirectory(path);
    if (_result.HasError())
    {
        throw _result;
    }
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      ファイルの移動・リネーム
 *  \param[in]  sourcePath          移動元のパス
 *  \param[in]  destPath            移動先のパス
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
void ThrowingUtility::Move(const PathView &sourcePath, const PathView &destPath)
{
    _result = Accessor::Move(sourcePath, destPath);
    if (_result.HasError())
    {
        throw _result;
    }
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      ファイルの削除
 *  \param[in]  path                削除するファイルのパス
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
void ThrowingUtility::Remove(const PathView &path)
{
    _result = Accessor::Remove(path);
    if (_result.HasError())
    {
        throw _result;
    }
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      ファイルのコピー
 *  \param[in]  sourcePath          コピー元のパス
 *  \param[in]  destPath            コピー先のパス
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
void ThrowingUtility::Copy(const PathView &sourcePath, const PathView &destPath)
{
    _result = Accessor::Copy(sourcePath, destPath);
    if (_result.HasError())
    {
        throw _result;
    }
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      ファイルの存在をチェック
 *  \param[in]  path                チェックするファイルのパス
 *  \retval     true                存在する
 *  \retval     false               存在しない
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
bool ThrowingUtility::Exists(const PathView &path)
{
    auto exists = Accessor::Exists(_result, path);
    if (_result.HasError())
    {
        throw _result;
    }

    return exists;
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      システム標準のファイルであるかをチェック
 *  \param[in]  path                パス
 *  \retval     true                システム標準のファイルである
 *  \retval     false               システム標準のファイルではない
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
bool ThrowingUtility::IsSystemNativeFile(const PathView &path)
{
    auto isSystemNativeFile = Accessor::IsSystemNativeFile(_result, path);
    if (_result.HasError())
    {
        throw _result;
    }

    return isSystemNativeFile;
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      マウント
 *  \param[in]  mountName           マウント名
 *  \param[in]  path                マウント先のパス
 *  \param[in]  accessType          アクセスタイプ
 *  \param[in]  delegateKey         デリゲートキー
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
void ThrowingUtility::Mount(const PathView &mountName,
                            const PathView &path,
                            MountAccessType accessType,
                            DelegateKey delegateKey)
{
    _result = Accessor::Mount(mountName, path, accessType, delegateKey);
    if (_result.HasError())
    {
        throw _result;
    }
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      マウント解除
 *  \param[in]  mountName           マウント名
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
void ThrowingUtility::Unmount(const PathView &mountName)
{
    _result = Accessor::Unmount(mountName);
    if (_result.HasError())
    {
        throw _result;
    }
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      再マウント
 *  \param[in]  mountName           マウント名
 *  \param[in]  path                マウント先のパス
 *  \param[in]  accessType          アクセスタイプ
 *  \param[in]  delegateKey         デリゲートキー
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
void ThrowingUtility::Remount(const PathView &mountName,
                              const PathView &path,
                              MountAccessType accessType,
                              DelegateKey delegateKey)
{
    _result = Accessor::Remount(mountName, path, accessType, delegateKey);
    if (_result.HasError())
    {
        throw _result;
    }
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      マウントされているかを取得
 *  \param[in]  mountName   マウント名
 *  \retval     true        マウントされている
 *  \retval     false       マウントされていない
 *  \note       この関数は例外を発生させず，File::IsMounted()と等価である
 */
/* ------------------------------------------------------------------------- */
bool ThrowingUtility::IsMounted(const PathView &mountName) noexcept
{
    return Accessor::IsMounted(mountName);
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      マウントパスからシステム標準のパスに変換
 *  \param[in]  path                パス
 *  \return     システム標準のパス
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 */
/* ------------------------------------------------------------------------- */
STL::string ThrowingUtility::GetSystemNativePath(const PathView &path)
{
    auto filesystemPath = Accessor::GetSystemNativePath(_result, path);

    if (_result.HasError())
    {
        throw _result;
    }

    return filesystemPath;
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      デリゲートの削除
 *  \param[in]  key                 削除するデリゲートキー
 *  \exception  MGL::File::Result   失敗時にエラーが設定されたリザルトを返す
 *  \note       存在しないデリゲートキーに対しては何もしない
 */
/* ------------------------------------------------------------------------- */
void ThrowingUtility::RemoveDelegate(DelegateKey key)
{
    if (_result = Accessor::RemoveDelegate(key); _result.HasError())
    {
        throw _result;
    }
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      デフォルトのデリゲートを設定
 *  \param[in]  key     デフォルトのデリゲートキー
 *  \note       この関数は例外を発生させず，File::SetDefaultDelegate()と等価である
 */
/* ------------------------------------------------------------------------- */
void ThrowingUtility::SetDefaultDelegate(DelegateKey key) noexcept
{
    Accessor::SetDefaultDelegate(key);
}
}    // namespace MGL::File

// vim: et ts=4 sw=4 sts=4
